#include <stdio.h>
#include <stdlib.h>

#include "linux/pkt_sched.h"
#include "linux/ip.h"

/* From wikipedia and other:
CS0   0x00  0  - 000 000
?
CS1   0x08  8  - 001 000
AF11  0x0A  10 - 001 010 - Low  Drop, Class1
AF12  0x0C  12 - 001 100 - Med  Drop, Class1
AF13  0x0E  14 - 001 110 - High Drop, Class1
CS2   0x0F  16 - 010 000
AF21  0x12  18 - 010 010 - Low  Drop, Class2
AF22  0x14  20 - 010 100 - Med  Drop, Class2
AF23  0x16  22 - 010 110 - High Drop, Class2
CS3   0x18  24 - 011 000
AF31  0x1A  26 - 011 010 - Low  Drop, Class3
AF32  0x1C  28 - 011 100 - Med  Drop, Class3
AF33  0x1E  30 - 011 110 - High Drop, Class3
CS4   0x20  32 - 100 000
AF41  0x22  34 - 100 010 - Low  Drop, Class4
AF42  0x24  36 - 100 100 - Med  Drop, Class4
AF43  0x26  38 - 100 110 - High Drop, Class4
CS5   0x28  40 - 101 000
?
VA    0x2C  44 - 101 100 - Voice Admit
EF    0x2E  46 - 101 110 - Expedited Forwarding
CS6   0x30  48 - 110 000
?
CS7   0x38 56 - 111 000
*/

/* from net/ipv4/route.c */
#define ECN_OR_COST(class)      TC_PRIO_##class

const __u8 ip_tos2prio[16] = {
        TC_PRIO_BESTEFFORT,
        ECN_OR_COST(BESTEFFORT),
        TC_PRIO_BESTEFFORT,
        ECN_OR_COST(BESTEFFORT),
        TC_PRIO_BULK,
        ECN_OR_COST(BULK),
        TC_PRIO_BULK,
        ECN_OR_COST(BULK),
        TC_PRIO_INTERACTIVE,
        ECN_OR_COST(INTERACTIVE),
        TC_PRIO_INTERACTIVE,
        ECN_OR_COST(INTERACTIVE),
        TC_PRIO_INTERACTIVE_BULK,
        ECN_OR_COST(INTERACTIVE_BULK),
        TC_PRIO_INTERACTIVE_BULK,
        ECN_OR_COST(INTERACTIVE_BULK)
};

/* from include/net/route.h */
static inline char rt_tos2priority(unsigned char tos)
{
    return ip_tos2prio[IPTOS_TOS(tos)>>1];
}


char dscp2tosprec(const char dscp)
{
    return dscp >> 3;
}

void dscpname(int dscp, char *name)
{
    if (0 == dscp) {
        sprintf(name, "CS0");
    }
    else if ( (dscp & 1)
             || (dscp < 8)
             || (dscp > 40) && (dscp < 44)
             || (dscp > 48) && (dscp < 56)
             || (dscp > 56))  {
        sprintf(name, "?%d?", dscp);
    }
    else if (! ((dscp & (1 << 1)) || (dscp & (1 << 2)) )) {
        sprintf(name, "CS%d", dscp >> 3);
    }
    else if ((dscp2tosprec(dscp) == 5)
             && (dscp & (1 << 2))
             && !(dscp & (1 << 1))) {
        sprintf(name, "VA");
    }
    else if ((dscp2tosprec(dscp) == 5)
             && (dscp & (1 << 2))
             && (dscp & (1 << 1))) {
        sprintf(name, "EF");
    }
    else {
        sprintf(name, "AF%d", dscp);
    }

}

const char *prio2name[] = {
    [TC_PRIO_BESTEFFORT] = "Best Effort",
    [TC_PRIO_BULK] = "Bulk",
    [TC_PRIO_INTERACTIVE] = "Interactive",
    [TC_PRIO_INTERACTIVE_BULK] = "Interactive Bulk"
};


int main(int argc, char** argv)
{
    int dscp;
    char dscp_name[5];

    printf("DSCP (10 | 16) | DSCP Name | TOS | Linux Prio\n");

    for (dscp = 0 ; dscp <= 63; dscp+=2) {
        dscpname(dscp, dscp_name);
        printf("%8d   %2X %12s %5d %3d %s\n",
               dscp, dscp, dscp_name, dscp2tosprec(dscp),
               rt_tos2priority(dscp), prio2name[rt_tos2priority(dscp)]);
    }
}
